@@ -106,6 +106,7 @@ impl StaticIndex<'_> {
                     type_hints: true,
                     parameter_hints: true,
                     chaining_hints: true,
+                    hide_named_constructor_hints: false,
                     max_length: Some(25),
                 },
                 file_id,
@@ -266,12 +267,10 @@ enum E { X(Foo) }
     fn derives() {
         check_all_ranges(
             r#"
+//- minicore:derive
 #[rustc_builtin_macro]
 pub macro Copy {}
         //^^^^
-#[rustc_builtin_macro]
-pub macro derive {}
-        //^^^^^^
 #[derive(Copy)]
 //^^^^^^ ^^^^
 struct Hello(i32);
